#include "pgqueue.h"

haPage* haPageQueue::get()
{
    mutex.lock();
    while (sendListHead == NULL) { 
	if (stopped) { 
	    mutex.unlock();
	    return NULL;
	}
	waiting = true;
	event.wait(mutex);
	waiting = false;
    }
    haPage* pg = sendListHead;
    sendListHead = pg->next;
    mutex.unlock();
    return pg;
}

void haPageQueue::put(int pageNo, int nPages, size_t offs, char* addr)
{
    mutex.lock();
    haPage* pg = freeList;
    if (pg == NULL) { 
	pg = new (pageSize) haPage();
    } else { 
	freeList = pg->next;
    }    
    pg->size = pageNo == 0 ? nPages*(pageSize+4) : 0;
    pg->offs = offs;
    memcpy(pg->body, addr, pageSize);
    pg->next = NULL;
    if (sendListHead != NULL) { 
	sendListTail->next = pg;
	sendListTail = pg;
    } else { 
	sendListTail = sendListHead = pg;
	if (waiting) { 
	    event.signal();
	}
    }
    mutex.unlock();
}

void haPageQueue::clear() 
{ 
    mutex.lock();
    if (sendListHead != NULL) { 
	sendListTail->next = freeList;
	freeList = sendListHead;
	sendListHead = sendListTail = NULL;
    }
    mutex.unlock();
}
    
void haPageQueue::free(haPage* pg) 
{ 
    mutex.lock();
    pg->next = freeList;
    freeList = pg;
    mutex.unlock();
}
    
void haPageQueue::stop()
{
    stopped = true;
    event.signal();
}

haPageQueue::~haPageQueue()
{
    haPage *pg, *next;
    for (pg = freeList; pg != NULL; pg = next) { 
	next = pg->next;
	delete pg;
    }
}





